library(psych) # useful for summary stats
library(kableExtra) # creates nice tables
library(survival)


df <- read.csv("D:/COURSES I HAVE RUN/Survival Analysis NCRM/copilot propodds dataset/sa_exercise.csv")
head(df)



head(df)

# Table of data
df %>%
  kbl() %>%
  kable_styling()

###
describe(df[ ,c("ses", "male", "time", "censor", "timedis")], fast=TRUE )


#variable for event is not there, but needs to be created using censor 
#event took place if case is not censored
df$event <- ifelse(df$censor == 1, 0, 1)
table (df$event, df$censor)

####***********************************************************************************************************
#### 1.Life tables for MAle covariate ####
####***********************************************************************************************************

#estimate the information for each subset based on Male covariate
lts1 <- survfit( Surv(timedis, event)~ 1, conf.type="none", subset=(male==0), data=df)
lts2 <- survfit( Surv(timedis, event)~ 1, conf.type="none", subset=(male==1), data=df)

# calculate the hazard functions
h1<-lts1$n.event/lts1$n.risk
h2<-lts2$n.event/lts2$n.risk

tab3_1<-data.frame(time=lts1$time, n.risk=lts1$n.risk, 
              n.event=lts1$n.event, n.censor=lts1$n.censor, 
              hazard=round(h1, 3), survival=round(lts1$surv, 3))

tab3_2<-data.frame(time=lts2$time, n.risk=lts2$n.risk, 
              n.event=lts2$n.event, n.censor=lts2$n.censor, 
              hazard=round(h2,3),  survival=round(lts2$surv, 3))


# Display the life table with hazard function
tab3_1 %>%
  kbl() %>%
  kable_classic_2(full_width = F)

tab3_2 %>%
  kbl() %>%
  kable_classic_2(full_width = F)




####***********************************************************************************************************
#### 2. Plot the hazard functions ####
####***********************************************************************************************************


library(ggplot2)
library("ggthemes") #to display plots based on preset templates
library(RColorBrewer) #to use preset colour combinations in the plots
library(dplyr) # to select some portions of data


#select the values I need for the plots
dth31<-select(tab3_1, c("time", "hazard"))
dth32<-select(tab3_2, c("time", "hazard"))

# Append values from the selection before
dthall<-rbind(dth31, dth32)
#create indicator for ses --> 
dthall$male <- c(rep(0, 6), rep(1, 6))


# Plot the discrete time hazard function with color palette
plot_hmale<-ggplot(dthall, aes(x = time, y = hazard, color = factor(male))) +
  geom_line(linewidth=3 ) +
  scale_color_brewer(palette="Set2",labels = c("Female", "Male")) +
  labs(title = "Hazard Function",
       x = "Time",
       y = "Hazard Function", 
       color = "Male") +
  ylim(c(0,.5)) +
  theme_hc() +  
  scale_x_continuous(breaks = seq(1, 6, by = 1)) #changes the ticks in x axis

plot_hmale



####***************************************************************
####***************************************************************
### 3.a Survivor function plot ####
####***************************************************************


# Create the survival function data frame
#select the values I need for the plots
dts31<-data.frame(time=lts1$time, survival=round(lts1$surv, 3))
dts32<-data.frame(time=lts2$time, survival=round(lts2$surv, 3))

# Append values from the selection before
dtsall<-rbind(dts31, dts32)
#create indicator for male 
dtsall$male <- c(rep(0, 6), rep(1, 6))


# Plot the survivor function
plot_surmale<-ggplot(dtsall, aes(x = time, y = survival, color = factor(male))) +
  ylim(c(0,1)) +
  geom_line(linewidth=3) +
  scale_color_brewer(palette="Set2",labels = c("Female", "Male")) +
  labs(title = "Survivor Function",
       x = "Time",
       y = "Survival prob.", 
       color = "Male") +
   theme_hc() +  
  scale_x_continuous(breaks = seq(1, 6, by = 1)) #changes the ticks in x axis

plot_surmale


####***************************************************************
####***************************************************************
### 3.b Median Lifetime ####
####***************************************************************

#************************************************* Females
# Extract survival probabilities and periods
surv_probs1 <- summary(lts1)$surv
surv_times1 <- summary(lts1)$time

# Find the time points around the median survival probability (0.5)
lower_index1 <- max(which(surv_probs1 >= 0.5))
upper_index1 <- min(which(surv_probs1 < 0.5))

# Linear interpolation to find the median lifetime
median_lifetime1 <- surv_times1[lower_index1] + 
  ((surv_probs1[lower_index1]-.5)  /
     (surv_probs1[lower_index1] - surv_probs1[upper_index1]))

# Print the median lifetime
print(median_lifetime1)

#************************************************Males 
#*# Extract survival probabilities and periods
surv_probs2 <- summary(lts2)$surv
surv_times2 <- summary(lts2)$time

# Find the time points around the median survival probability (0.5)
lower_index2 <- max(which(surv_probs2 >= 0.5))
upper_index2 <- min(which(surv_probs2 < 0.5))

# Linear interpolation to find the median lifetime
median_lifetime2 <- surv_times2[lower_index2] + 
  ((surv_probs2[lower_index2]-.5)  /
     (surv_probs2[lower_index2] - surv_probs2[upper_index2]))

# Print the median lifetime
print(median_lifetime2)


#*********************************************

median_lifetimes <- data.frame(
  male = c("Female", "Male"),
  median_lifetime = c(2.88, 2.50) 
)
####***************************************************************
####***************************************************************
### 3.c Survival Curve with median lifetimes ####
####***************************************************************
# Add annotations for each male group
for (i in 1:nrow(median_lifetimes)) {
  plot_surmale <- plot_surmale +
    geom_vline(xintercept = median_lifetimes$median_lifetime[i], linetype = "dashed", color = scales::brewer_pal(palette = "Set2")(3)[i]) +
    annotate("text", x = median_lifetimes$median_lifetime[i], y = 0.5, 
             label = paste("", median_lifetimes$median_lifetime[i]), 
             vjust = -5, color = scales::brewer_pal(palette = "Set2")(3)[i])
}

# Print the plot
print(plot_surmale)


####***************************************************************
####***************************************************************
### Alternative approach Survival Curve: use the survminer package ####
####***************************************************************

library(survminer) # used to create survival analysis plots


# Fit the survival model
surv_fit <- survfit(Surv(timedis, event) ~ male, data = df) #the ~ses is including this covariate

# Plot the survival curves with 95% confidence intervals
ggsurvplot(
  surv_fit,
  data = df,
  conf.int = TRUE,
  pval = TRUE,
  risk.table = TRUE,
  legend.labs = c("Female", "Male"),
  legend.title = "Gender",
  xlab = "Time",
  ylab = "Survival Probability",
  ggtheme = theme_minimal()
)

####***************************************************************
####***************************************************************
### 4.a Odds transormation ####
####***************************************************************

# odds transformation
dthall$oddsh<-dthall$hazard/(1-dthall$hazard)


# Plot the discrete time odds with colour palette
plot_oddmale<-ggplot(dthall, aes(x = time, y = oddsh, color = factor(male))) +
  geom_line(linewidth=3 ) +
  scale_color_brewer(palette="Set2",labels = c("Female", "Male")) +
  labs(title = "Odds of Hazard Function",
       x = "Time",
       y = "Odds of Event", 
       color = "Gender") +
  ylim(c(0,1)) +
  theme_hc() 

plot_oddmale


####***************************************************************
####***************************************************************
### 4.b Logit ####
####***************************************************************

# logits  transformation
dthall$logith<-log(dthall$oddsh) #the natural log of the odds 




# Plot the discrete time hazard function with colour palette
plot_logitmale<-ggplot(dthall, aes(x = time, y = logith, color = factor(male))) +
  geom_line(linewidth=3 ) +
  scale_color_brewer(palette="Set2",labels = c("Female", "Male")) +
 labs(title = "Logits of Hazard Function",
       x = "Time",
       y = "Odds of Event", 
       color = "Gender") +
  ylim(-4, 0) +
  theme_hc() 

plot_logitmale

####***************************************************************
####***************************************************************
### 4.c Arrange the plots in one figure ####
####***************************************************************

library(ggpubr) #allows to arrange plots together
figure <- ggarrange(plot_hmale, plot_oddmale, plot_logitmale, 
                    ncol = 2, nrow = 2, common.legend = TRUE, legend="top")
figure

####***************************************************************
####***************************************************************
### Restructure data to Person-Period  ####
####***************************************************************

library(tidyverse) # to manipulate dataset

# Function to create person-period data
create_person_period <- function(data) {
  data %>%
    rowwise() %>%
    mutate(periods = list(1:timedis)) %>%
    unnest(periods) %>%
    mutate(event = ifelse(periods == timedis & event == 1, 1, 0),
           censor = ifelse(periods == timedis & censor == 1, 1, 0)) %>%
    filter(periods <= timedis) %>%
    select(ID, periods, event, censor, male, ses)
}


#Explanation:
# 1.mutate(periods = list(1:timedis)):
#This creates a list of periods from 1 to the event time or censoring time.
#2.unnest(periods):
# This expands the list of periods into individual rows.
#3.mutate(event = ifelse(periods == timedis & event == 1, 1, 0), 
#censor = ifelse(periods == timedis & censor == 1, 1, 0)):
# This ensures that the event and censor columns are correctly marked 
#only for the period when the event or censoring occurs.
#4. filter(periods <= timedis):
#This filters out any periods beyond the event or censoring time, 
#ensuring that individuals only contribute periods up to the event or censoring time.
#This should ensure that each individual only contributes periods up to 
#the event or censoring time, and no additional periods are included.

# Apply the function to the dataset
ppdf <- create_person_period(df)


# Table of data
ppdf %>%
  kbl() %>%
  kable_styling()


####***************************************************************
####***************************************************************
### 5. General Specification Model & Covariate Male ####
####***************************************************************


m1<-glm(event~factor(periods) + male - 1, family="binomial", data=ppdf)
summary(m1)


#estimated odds ratios associated with male covariate:
exp(0.2586)
#estimated 95% confience interval of the slope of male, based on SE:
0.2586-(1.96*0.1248)
0.2586+(1.96*0.1248)
#estimated 95% confidence interval of the odds ratios for male covariate:
exp(0.2586-(1.96*0.1248))
exp(0.2586+(1.96*0.1248))

#create summaries for plotting
t<-data.frame(hazard=m1$fitted.values, time=ppdf$periods, male=ppdf$male)
t$logit<-log(t$hazard/(1-t$hazard))

ta<-aggregate(t, list(male=t$male, time=t$time),mean)
ta.0<-ta[ta$male==0, ]
ta.1<-ta[ta$male==1, ]





####***************************************************************
####***************************************************************
### Plot the two lines
####***************************************************************

# Create a data frame for the new line


baseline<-select(ta.0, c("time", "logit", "male"))
covariate<-select(ta.1, c("time", "logit", "male"))
dfm1<-rbind(baseline, covariate)

# Plot the lines


# Plot the discrete time hazard function with color palette
plot_m1<-ggplot(dfm1, aes(x = time, y = logit, color = factor(male))) +
  geom_line(linewidth=3 ) +
  scale_color_manual(values = c("#357EBD", "#EEA236"), labels = c("Female", "Male")) +
  #scale_color_brewer(palette="Paired",labels = c("No", "Yes")) + # Not used here, I opted for manually assigned colours above
  labs(title = "Estimated Logits of Hazard Function",
       x = "Time",
       y = "Estimted Values", 
       color = "Sex") +
  ylim(-4, 0) +
  theme_hc() 

plot_m1



####***************************************************************
####***************************************************************
### Plot the model in different metrics --> Odds and Hazard (prob.) 
####***************************************************************

print(dfm1) 

# dfmodB is the data frame with the logits estimated in model B
# I will use transformations to get the odds and the hazard function in probability scale

dfm1$odds<-exp(dfm1$logit)
dfm1$hf<-1/(1+exp(-dfm1$logit))



# Plot the odds
plot_m1odds<-ggplot(dfm1, aes(x = time, y = odds, color = factor(male))) +
  geom_line(size=3 ) +
  scale_color_manual(values = c("#357EBD", "#EEA236"), labels = c("Female", "Male")) +
  labs(title = "Estimated Odds of Hazard Function",
       x = "Time",
       y = "Estimated Values", 
       color = "Sex") +
  ylim(0, 1) +
  theme_hc() 

plot_m1odds


# Plot the Hazard estimates
plot_m1hf<-ggplot(dfm1, aes(x = time, y = hf, color = factor(male))) +
  geom_line(size=3 ) +
  scale_color_manual(values = c("#357EBD", "#EEA236"), labels = c("Female", "Male")) +
  labs(title = "Estimated Hazard Function",
       x = "Time",
       y = "Estimated Values", 
       color = "Sex") +
  ylim(0, .6) +
  theme_hc() 

plot_m1hf


library(ggpubr) #allows to arrange plots together
figure <- ggarrange(plot_m1, plot_m1odds, plot_m1hf, 
                    ncol = 2, nrow = 2, common.legend = TRUE, legend="bottom")
figure



####***************************************************************
####***************************************************************
### 6. Linear effect of period and Covariate Male ####
####***************************************************************

describe(ppdf)
#center the periods variable: period 1 is the reference period:
ppdf$cenperiod<-ppdf$periods-1
table(ppdf$cenperiod, ppdf$periods)

mlin<-glm(event~cenperiod + male , family="binomial", data=ppdf)
summary(mlin)

### hazard of intercept
1/(1+exp(-(-1.53754)))
